@@ -1,4 +1,4 @@
-;;; haskell-mode-autoloads.el --- automatically extracted autoloads
+;;; haskell-mode-autoloads.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
@@ -6,8 +6,6 @@
 ;;;### (autoloads nil "ghc-core" "ghc-core.el" (0 0 0 0))
 ;;; Generated autoloads from ghc-core.el
 
-(let ((loads (get 'ghc-core 'custom-loads))) (if (member '"ghc-core" loads) nil (put 'ghc-core 'custom-loads (cons '"ghc-core" loads))))
-
 (autoload 'ghc-core-create-core "ghc-core" "\
 Compile and load the current buffer as tidy core." t nil)
 
@@ -20,7 +18,7 @@ Major mode for GHC Core files.
 
 \(fn)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "ghc-core" '("ghc-core-")))
+(register-definition-prefixes "ghc-core" '("ghc-core-"))
 
 ;;;***
 
@@ -35,7 +33,7 @@ Major mode for working with .ghci files.
 
 (add-to-list 'auto-mode-alist '("\\.ghci\\'" . ghci-script-mode))
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "ghci-script-mode" '("ghci-script-mode-")))
+(register-definition-prefixes "ghci-script-mode" '("ghci-script-mode-"))
 
 ;;;***
 
@@ -45,10 +43,20 @@ Major mode for working with .ghci files.
 (autoload 'interactive-haskell-mode "haskell" "\
 Minor mode for enabling haskell-process interaction.
 
-If called interactively, enable Interactive-Haskell mode if ARG
-is positive, and disable it if ARG is zero or negative.  If
-called from Lisp, also enable the mode if ARG is omitted or nil,
-and toggle it if ARG is `toggle'; disable the mode otherwise.
+This is a minor mode.  If called interactively, toggle the
+`Interactive-Haskell mode' mode.  If the prefix argument is
+positive, enable the mode, and if it is zero or negative, disable
+the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable
+the mode if ARG is nil, omitted, or is a positive number.
+Disable the mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `interactive-haskell-mode'.
+
+The mode's hook is called both when the mode is enabled and when
+it is disabled.
 
 \(fn &optional ARG)" t nil)
 
@@ -127,7 +135,7 @@ Prompts for a Cabal command to run.
 (autoload 'haskell-process-minimal-imports "haskell" "\
 Dump minimal imports." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell" '("haskell-" "interactive-haskell-mode-map" "xref-prompt-for-identifier")))
+(register-definition-prefixes "haskell" '("haskell-" "interactive-haskell-mode-map" "xref-prompt-for-identifier"))
 
 ;;;***
 
@@ -138,7 +146,7 @@ Dump minimal imports." t nil)
 (autoload 'haskell-align-imports "haskell-align-imports" "\
 Align all the imports in the buffer." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-align-imports" '("haskell-align-imports-")))
+(register-definition-prefixes "haskell-align-imports" '("haskell-align-imports-"))
 
 ;;;***
 
@@ -152,7 +160,7 @@ Mode for editing *.chs files of the c2hs haskell tool.
 
 \(fn)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-c2hs" '("haskell-c2hs-font-lock-keywords")))
+(register-definition-prefixes "haskell-c2hs" '("haskell-c2hs-font-lock-keywords"))
 
 ;;;***
 
@@ -175,8 +183,9 @@ file), then this function returns nil.
 \(fn NAME)" t nil)
 
 (autoload 'haskell-cabal-get-dir "haskell-cabal" "\
-Get the Cabal dir for a new project. Various ways of figuring this out,
-   and indeed just prompting the user. Do them all.
+Get the Cabal dir for a new project.
+Various ways of figuring this out, and indeed just prompting the user.  Do them
+all.
 
 \(fn &optional USE-DEFAULTS)" nil nil)
 
@@ -191,9 +200,7 @@ OTHER-WINDOW use `find-file-other-window'.
 
 \(fn OTHER-WINDOW)" t nil)
 
-(let ((loads (get 'haskell-cabal 'custom-loads))) (if (member '"haskell-cabal" loads) nil (put 'haskell-cabal 'custom-loads (cons '"haskell-cabal" loads))))
-
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-cabal" '("haskell-")))
+(register-definition-prefixes "haskell-cabal" '("haskell-"))
 
 ;;;***
 
@@ -204,14 +211,24 @@ OTHER-WINDOW use `find-file-other-window'.
 (autoload 'haskell-collapse-mode "haskell-collapse" "\
 Minor mode to collapse and expand haskell expressions
 
-If called interactively, enable Haskell-Collapse mode if ARG is
-positive, and disable it if ARG is zero or negative.  If called
-from Lisp, also enable the mode if ARG is omitted or nil, and
-toggle it if ARG is `toggle'; disable the mode otherwise.
+This is a minor mode.  If called interactively, toggle the
+`Haskell-Collapse mode' mode.  If the prefix argument is
+positive, enable the mode, and if it is zero or negative, disable
+the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable
+the mode if ARG is nil, omitted, or is a positive number.
+Disable the mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `haskell-collapse-mode'.
+
+The mode's hook is called both when the mode is enabled and when
+it is disabled.
 
 \(fn &optional ARG)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-collapse" '("haskell-")))
+(register-definition-prefixes "haskell-collapse" '("haskell-"))
 
 ;;;***
 
@@ -322,7 +339,7 @@ cursor position and markers by using
 (autoload 'haskell-mode-find-uses "haskell-commands" "\
 Find use cases of the identifier at point and highlight them all." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-commands" '("haskell-")))
+(register-definition-prefixes "haskell-commands" '("haskell-"))
 
 ;;;***
 
@@ -330,8 +347,6 @@ Find use cases of the identifier at point and highlight them all." t nil)
 ;;;;;;  0 0))
 ;;; Generated autoloads from haskell-compile.el
 
-(let ((loads (get 'haskell-compile 'custom-loads))) (if (member '"haskell-compile" loads) nil (put 'haskell-compile 'custom-loads (cons '"haskell-compile" loads))))
-
 (autoload 'haskell-compile "haskell-compile" "\
 Run a compile command for the current Haskell buffer.
 Obeys haskell-compiler-type to choose the appropriate build command.
@@ -356,7 +371,7 @@ base directory for build tools, or the current buffer for
 
 \(fn &optional EDIT-COMMAND)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-compile" '("haskell-")))
+(register-definition-prefixes "haskell-compile" '("haskell-"))
 
 ;;;***
 
@@ -364,7 +379,7 @@ base directory for build tools, or the current buffer for
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-complete-module.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-complete-module" '("haskell-complete-module")))
+(register-definition-prefixes "haskell-complete-module" '("haskell-complete-module"))
 
 ;;;***
 
@@ -372,15 +387,13 @@ base directory for build tools, or the current buffer for
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-completions.el
 
-(let ((loads (get 'haskell-completions 'custom-loads))) (if (member '"haskell-completions" loads) nil (put 'haskell-completions 'custom-loads (cons '"haskell-completions" loads))))
-
 (autoload 'haskell-completions-completion-at-point "haskell-completions" "\
 Provide completion list for thing at point.
 This function is used in non-interactive `haskell-mode'.  It
 provides completions for haskell keywords, language pragmas,
 GHC's options, and language extensions, but not identifiers." nil nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-completions" '("haskell-completions-")))
+(register-definition-prefixes "haskell-completions" '("haskell-completions-"))
 
 ;;;***
 
@@ -388,38 +401,14 @@ GHC's options, and language extensions, but not identifiers." nil nil)
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-customize.el
 
-(let ((loads (get 'haskell 'custom-loads))) (if (member '"haskell-customize" loads) nil (put 'haskell 'custom-loads (cons '"haskell-customize" loads))))
-
-(let ((loads (get 'haskell-interactive 'custom-loads))) (if (member '"haskell-customize" loads) nil (put 'haskell-interactive 'custom-loads (cons '"haskell-customize" loads))))
-
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-customize" '("haskell-" "inferior-haskell-root-dir")))
+(register-definition-prefixes "haskell-customize" '("haskell-" "inferior-haskell-root-dir"))
 
 ;;;***
 
 ;;;### (autoloads nil "haskell-debug" "haskell-debug.el" (0 0 0 0))
 ;;; Generated autoloads from haskell-debug.el
 
-(let ((loads (get 'haskell-debug 'custom-loads))) (if (member '"haskell-debug" loads) nil (put 'haskell-debug 'custom-loads (cons '"haskell-debug" loads))))
-
-(defface haskell-debug-warning-face '((t :inherit 'compilation-warning)) "\
-Face for warnings." :group 'haskell-debug)
-
-(defface haskell-debug-trace-number-face '((t :weight bold :background "#f5f5f5")) "\
-Face for numbers in backtrace." :group 'haskell-debug)
-
-(defface haskell-debug-newline-face '((t :weight bold :background "#f0f0f0")) "\
-Face for newlines in trace steps." :group 'haskell-debug)
-
-(defface haskell-debug-keybinding-face '((t :inherit 'font-lock-type-face :weight bold)) "\
-Face for keybindings." :group 'haskell-debug)
-
-(defface haskell-debug-heading-face '((t :inherit 'font-lock-keyword-face)) "\
-Face for headings." :group 'haskell-debug)
-
-(defface haskell-debug-muted-face '((t :foreground "#999")) "\
-Face for muteds." :group 'haskell-debug)
-
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-debug" '("haskell-debug")))
+(register-definition-prefixes "haskell-debug" '("haskell-debug"))
 
 ;;;***
 
@@ -427,8 +416,6 @@ Face for muteds." :group 'haskell-debug)
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-decl-scan.el
 
-(let ((loads (get 'haskell-decl-scan 'custom-loads))) (if (member '"haskell-decl-scan" loads) nil (put 'haskell-decl-scan 'custom-loads (cons '"haskell-decl-scan" loads))))
-
 (autoload 'haskell-ds-create-imenu-index "haskell-decl-scan" "\
 Function for finding `imenu' declarations in Haskell mode.
 Finds all declarations (classes, variables, imports, instances and
@@ -475,18 +462,57 @@ Invokes `haskell-decl-scan-mode-hook' on activation.
 
 \(fn &optional ARG)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-decl-scan" '("haskell-d" "literate-haskell-ds-")))
+(register-definition-prefixes "haskell-decl-scan" '("haskell-d" "literate-haskell-ds-"))
 
 ;;;***
 
 ;;;### (autoloads nil "haskell-doc" "haskell-doc.el" (0 0 0 0))
 ;;; Generated autoloads from haskell-doc.el
 
-(let ((loads (get 'haskell-doc 'custom-loads))) (if (member '"haskell-doc" loads) nil (put 'haskell-doc 'custom-loads (cons '"haskell-doc" loads))))
-
 (autoload 'haskell-doc-mode "haskell-doc" "\
 Enter `haskell-doc-mode' for showing fct types in the echo area.
-See variable docstring.
+
+This is a minor mode.  If called interactively, toggle the
+`Haskell-Doc mode' mode.  If the prefix argument is positive,
+enable the mode, and if it is zero or negative, disable the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable
+the mode if ARG is nil, omitted, or is a positive number.
+Disable the mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `haskell-doc-mode'.
+
+The mode's hook is called both when the mode is enabled and when
+it is disabled.
+
+When enabled, shows the type of the function near point or a related comment.
+
+If the identifier near point is a Haskell keyword and the variable
+`haskell-doc-show-reserved' is non-nil show a one line summary
+of the syntax.
+
+If the identifier near point is a Prelude or one of the standard library
+functions and `haskell-doc-show-prelude' is non-nil show its type.
+
+If the identifier near point is local (i.e. defined in this module) check
+the `imenu' list of functions for the type.  This obviously requires that
+your language mode uses `imenu'.
+
+If the identifier near point is global (i.e. defined in an imported module)
+and the variable `haskell-doc-show-global-types' is non-nil show the type of its
+function.
+
+If the identifier near point is a standard strategy or a function, type related
+related to strategies and `haskell-doc-show-strategy' is non-nil show the type
+of the function.  Strategies are special to the parallel execution of Haskell.
+If you're not interested in that just turn it off.
+
+If the identifier near point is a user defined function that occurs as key
+in the alist `haskell-doc-user-defined-ids' and the variable
+`haskell-doc-show-user-defined' is non-nil show the type of the function.
+
+This variable is buffer-local.
 
 \(fn &optional ARG)" t nil)
 
@@ -494,6 +520,16 @@ See variable docstring.
 
 (defalias 'turn-on-haskell-doc 'haskell-doc-mode)
 
+(autoload 'haskell-doc-eldoc-function "haskell-doc" "\
+Function for use by eldoc.
+
+By accepting CALLBACK, it is designed to be used in
+`eldoc-documentation-functions' in Emacs >= 28.1, but by making
+that argument optional it can also be set directly as
+`eldoc-documentation-function' in older Emacsen.
+
+\(fn &optional CALLBACK)" nil nil)
+
 (autoload 'haskell-doc-current-info "haskell-doc" "\
 Return the info about symbol at point.
 Meant for `eldoc-documentation-function'." nil nil)
@@ -507,7 +543,7 @@ current buffer.
 
 \(fn &optional SYM)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-doc" '("haskell-" "inferior-haskell-" "turn-off-haskell-doc")))
+(register-definition-prefixes "haskell-doc" '("haskell-" "inferior-haskell-" "turn-off-haskell-doc"))
 
 ;;;***
 
@@ -515,31 +551,7 @@ current buffer.
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-font-lock.el
 
-(let ((loads (get 'haskell-appearance 'custom-loads))) (if (member '"haskell-font-lock" loads) nil (put 'haskell-appearance 'custom-loads (cons '"haskell-font-lock" loads))))
-
-(defface haskell-keyword-face '((t :inherit font-lock-keyword-face)) "\
-Face used to highlight Haskell keywords." :group 'haskell-appearance)
-
-(defface haskell-type-face '((t :inherit font-lock-type-face)) "\
-Face used to highlight Haskell types" :group 'haskell-appearance)
-
-(defface haskell-constructor-face '((t :inherit font-lock-type-face)) "\
-Face used to highlight Haskell constructors." :group 'haskell-appearance)
-
-(defface haskell-operator-face '((t :inherit font-lock-variable-name-face)) "\
-Face used to highlight Haskell operators." :group 'haskell-appearance)
-
-(defface haskell-pragma-face '((t :inherit font-lock-preprocessor-face)) "\
-Face used to highlight Haskell pragmas ({-# ... #-})." :group 'haskell-appearance)
-
-(defface haskell-liquid-haskell-annotation-face '((t :inherit haskell-pragma-face)) "\
-Face used to highlight LiquidHaskell annotations ({-@ ... @-})." :group 'haskell-appearance)
-
-(defface haskell-literate-comment-face '((t :inherit font-lock-doc-face)) "\
-Face with which to fontify literate comments.
-Inherit from `default' to avoid fontification of them." :group 'haskell-appearance)
-
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-font-lock" '("haskell-")))
+(register-definition-prefixes "haskell-font-lock" '("haskell-"))
 
 ;;;***
 
@@ -547,7 +559,7 @@ Inherit from `default' to avoid fontification of them." :group 'haskell-appearan
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-ghc-support.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-ghc-support" '("haskell-")))
+(register-definition-prefixes "haskell-ghc-support" '("haskell-"))
 
 ;;;***
 
@@ -573,7 +585,7 @@ Lookup QUERY at `haskell-hoogle-url'.
 (autoload 'haskell-hoogle-lookup-from-local "haskell-hoogle" "\
 Lookup QUERY on local hoogle server." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-hoogle" '("haskell-hoogle-" "hoogle-prompt")))
+(register-definition-prefixes "haskell-hoogle" '("haskell-hoogle-" "hoogle-prompt"))
 
 ;;;***
 
@@ -581,8 +593,6 @@ Lookup QUERY on local hoogle server." t nil)
 ;;;;;;  0))
 ;;; Generated autoloads from haskell-indent.el
 
-(let ((loads (get 'haskell-indent 'custom-loads))) (if (member '"haskell-indent" loads) nil (put 'haskell-indent 'custom-loads (cons '"haskell-indent" loads))))
-
 (autoload 'turn-on-haskell-indent "haskell-indent" "\
 Turn on ``intelligent'' Haskell indentation mode." nil nil)
 
@@ -615,7 +625,7 @@ Invokes `haskell-indent-hook' if not nil.
 
 \(fn &optional ARG)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-indent" '("haskell-indent-" "turn-off-haskell-indent")))
+(register-definition-prefixes "haskell-indent" '("haskell-indent-" "turn-off-haskell-indent"))
 
 ;;;***
 
@@ -623,24 +633,32 @@ Invokes `haskell-indent-hook' if not nil.
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-indentation.el
 
-(let ((loads (get 'haskell-indentation 'custom-loads))) (if (member '"haskell-indentation" loads) nil (put 'haskell-indentation 'custom-loads (cons '"haskell-indentation" loads))))
-
 (autoload 'haskell-indentation-mode "haskell-indentation" "\
 Haskell indentation mode that deals with the layout rule.
 It rebinds RET, DEL and BACKSPACE, so that indentations can be
 set and deleted as if they were real tabs.
 
-If called interactively, enable Haskell-Indentation mode if ARG
-is positive, and disable it if ARG is zero or negative.  If
-called from Lisp, also enable the mode if ARG is omitted or nil,
-and toggle it if ARG is `toggle'; disable the mode otherwise.
+This is a minor mode.  If called interactively, toggle the
+`Haskell-Indentation mode' mode.  If the prefix argument is
+positive, enable the mode, and if it is zero or negative, disable
+the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable
+the mode if ARG is nil, omitted, or is a positive number.
+Disable the mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `haskell-indentation-mode'.
+
+The mode's hook is called both when the mode is enabled and when
+it is disabled.
 
 \(fn &optional ARG)" t nil)
 
 (autoload 'turn-on-haskell-indentation "haskell-indentation" "\
 Turn on the haskell-indentation minor mode." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-indentation" '("haskell-indentation-")))
+(register-definition-prefixes "haskell-indentation" '("haskell-indentation-"))
 
 ;;;***
 
@@ -648,26 +666,6 @@ Turn on the haskell-indentation minor mode." t nil)
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-interactive-mode.el
 
-(defface haskell-interactive-face-prompt '((t :inherit font-lock-function-name-face)) "\
-Face for the prompt." :group 'haskell-interactive)
-
-(defface haskell-interactive-face-prompt-cont '((t :inherit font-lock-keyword-face)) "\
-Face for GHCi's prompt-cont in multi-line mode." :group 'haskell-interactive)
-
-(define-obsolete-face-alias 'haskell-interactive-face-prompt2 'haskell-interactive-face-prompt-cont "16.2")
-
-(defface haskell-interactive-face-compile-error '((t :inherit compilation-error)) "\
-Face for compile errors." :group 'haskell-interactive)
-
-(defface haskell-interactive-face-compile-warning '((t :inherit compilation-warning)) "\
-Face for compiler warnings." :group 'haskell-interactive)
-
-(defface haskell-interactive-face-result '((t :inherit font-lock-string-face)) "\
-Face for the result." :group 'haskell-interactive)
-
-(defface haskell-interactive-face-garbage '((t :inherit font-lock-string-face)) "\
-Face for trailing garbage after a command has completed." :group 'haskell-interactive)
-
 (autoload 'haskell-interactive-mode-reset-error "haskell-interactive-mode" "\
 Reset the error cursor position.
 
@@ -686,7 +684,7 @@ function `haskell-presentation-present', depending on variable
 
 \(fn LINE)" nil nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-interactive-mode" '("haskell-")))
+(register-definition-prefixes "haskell-interactive-mode" '("haskell-"))
 
 ;;;***
 
@@ -694,22 +692,13 @@ function `haskell-presentation-present', depending on variable
 ;;;;;;  0))
 ;;; Generated autoloads from haskell-lexeme.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-lexeme" '("haskell-lexeme-")))
+(register-definition-prefixes "haskell-lexeme" '("haskell-lexeme-"))
 
 ;;;***
 
 ;;;### (autoloads nil "haskell-load" "haskell-load.el" (0 0 0 0))
 ;;; Generated autoloads from haskell-load.el
 
-(defface haskell-error-face '((((supports :underline (:style wave))) :underline (:style wave :color "#dc322f")) (t :inherit error)) "\
-Face used for marking error lines." :group 'haskell-mode)
-
-(defface haskell-warning-face '((((supports :underline (:style wave))) :underline (:style wave :color "#b58900")) (t :inherit warning)) "\
-Face used for marking warning lines." :group 'haskell-mode)
-
-(defface haskell-hole-face '((((supports :underline (:style wave))) :underline (:style wave :color "#6c71c4")) (t :inherit warning)) "\
-Face used for marking hole lines." :group 'haskell-mode)
-
 (autoload 'haskell-process-reload-devel-main "haskell-load" "\
 Reload the module `DevelMain' and then run `DevelMain.update'.
 
@@ -719,7 +708,7 @@ applications.  Put your development version of the program in
 new thread, and use the `foreign-store' package to access the
 running context across :load/:reloads in GHCi." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-load" '("haskell-")))
+(register-definition-prefixes "haskell-load" '("haskell-"))
 
 ;;;***
 
@@ -729,7 +718,7 @@ running context across :load/:reloads in GHCi." t nil)
 (autoload 'haskell-menu "haskell-menu" "\
 Launch the Haskell sessions menu." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-menu" '("haskell-menu-")))
+(register-definition-prefixes "haskell-menu" '("haskell-menu-"))
 
 ;;;***
 
@@ -779,17 +768,18 @@ Other modes:
       Scans top-level declarations, and places them in a menu.
 
     `haskell-doc-mode', Hans-Wolfgang Loidl
-      Echoes types of functions or syntax of keywords when the cursor is idle.
+      Sets up eldoc to echo types of functions or syntax of keywords
+      when the cursor is idle.
 
 To activate a minor-mode, simply run the interactive command. For
 example, `M-x haskell-doc-mode'. Run it again to disable it.
 
-To enable a mode for every haskell-mode buffer, add a hook in
+To enable a mode for every `haskell-mode' buffer, add a hook in
 your Emacs configuration. To do that you can customize
 `haskell-mode-hook' or add lines to your .emacs file. For
 example, to enable `interactive-haskell-mode', use the following:
 
-    (add-hook 'haskell-mode-hook 'interactive-haskell-mode)
+    (add-hook \\='haskell-mode-hook \\='interactive-haskell-mode)
 
 Minor modes that work well with `haskell-mode':
 
@@ -842,7 +832,7 @@ generated.
 
 \(fn &optional AND-THEN-FIND-THIS-TAG)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-mode" '("haskell-")))
+(register-definition-prefixes "haskell-mode" '("haskell-"))
 
 ;;;***
 
@@ -867,7 +857,7 @@ If DONTCREATE is non-nil don't create a new session.
 
 \(fn SESSION &optional DONTCREATE)" nil nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-modules" '("haskell-")))
+(register-definition-prefixes "haskell-modules" '("haskell-"))
 
 ;;;***
 
@@ -876,7 +866,9 @@ If DONTCREATE is non-nil don't create a new session.
 ;;; Generated autoloads from haskell-move-nested.el
 
 (autoload 'haskell-move-nested "haskell-move-nested" "\
-Shift the nested off-side-rule block adjacent to point by COLS columns to the right.
+Shift the nested off-side-rule block adjacent to point.
+It shift the nested off-side-rule block adjacent to point by COLS
+columns to the right.
 
 In Transient Mark mode, if the mark is active, operate on the contents
 of the region instead.
@@ -903,7 +895,7 @@ of the region instead.
 
 \(fn COLS)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-move-nested" '("haskell-")))
+(register-definition-prefixes "haskell-move-nested" '("haskell-"))
 
 ;;;***
 
@@ -923,7 +915,7 @@ Go to the first line of a list of consecutive import lines. Cycles." t nil)
 Return to the non-import point we were at before going to the module list.
    If we were originally at an import list, we can just cycle through easily." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-navigate-imports" '("haskell-navigate-imports-")))
+(register-definition-prefixes "haskell-navigate-imports" '("haskell-navigate-imports-"))
 
 ;;;***
 
@@ -931,7 +923,7 @@ Return to the non-import point we were at before going to the module list.
 ;;;;;;  (0 0 0 0))
 ;;; Generated autoloads from haskell-presentation-mode.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-presentation-mode" '("haskell-presentation-")))
+(register-definition-prefixes "haskell-presentation-mode" '("haskell-presentation-"))
 
 ;;;***
 
@@ -939,14 +931,14 @@ Return to the non-import point we were at before going to the module list.
 ;;;;;;  0 0))
 ;;; Generated autoloads from haskell-process.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-process" '("haskell-")))
+(register-definition-prefixes "haskell-process" '("haskell-"))
 
 ;;;***
 
 ;;;### (autoloads nil "haskell-repl" "haskell-repl.el" (0 0 0 0))
 ;;; Generated autoloads from haskell-repl.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-repl" '("haskell-interactive-")))
+(register-definition-prefixes "haskell-repl" '("haskell-interactive-"))
 
 ;;;***
 
@@ -954,7 +946,7 @@ Return to the non-import point we were at before going to the module list.
 ;;;;;;  0 0))
 ;;; Generated autoloads from haskell-sandbox.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-sandbox" '("haskell-sandbox-")))
+(register-definition-prefixes "haskell-sandbox" '("haskell-sandbox-"))
 
 ;;;***
 
@@ -970,7 +962,7 @@ Get the session process.
 
 \(fn S)" nil nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-session" '("haskell-session")))
+(register-definition-prefixes "haskell-session" '("haskell-session"))
 
 ;;;***
 
@@ -985,7 +977,7 @@ i.e. an import list separated by blank lines on either side.
 If the region is active, it will restrict the imports to sort
 within that region." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-sort-imports" '("haskell-sort-imports-")))
+(register-definition-prefixes "haskell-sort-imports" '("haskell-sort-imports-"))
 
 ;;;***
 
@@ -993,14 +985,14 @@ within that region." t nil)
 ;;;;;;  0))
 ;;; Generated autoloads from haskell-string.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-string" '("haskell-")))
+(register-definition-prefixes "haskell-string" '("haskell-"))
 
 ;;;***
 
 ;;;### (autoloads nil "haskell-svg" "haskell-svg.el" (0 0 0 0))
 ;;; Generated autoloads from haskell-svg.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-svg" '("haskell-svg-")))
+(register-definition-prefixes "haskell-svg" '("haskell-svg-"))
 
 ;;;***
 
@@ -1018,7 +1010,7 @@ Set input method `haskell-unicode'." t nil)
 ;;;### (autoloads nil "haskell-utils" "haskell-utils.el" (0 0 0 0))
 ;;; Generated autoloads from haskell-utils.el
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "haskell-utils" '("haskell-")))
+(register-definition-prefixes "haskell-utils" '("haskell-"))
 
 ;;;***
 
@@ -1029,36 +1021,40 @@ Set input method `haskell-unicode'." t nil)
 (autoload 'highlight-uses-mode "highlight-uses-mode" "\
 Minor mode for highlighting and jumping between uses.
 
-If called interactively, enable Highlight-Uses mode if ARG is
-positive, and disable it if ARG is zero or negative.  If called
-from Lisp, also enable the mode if ARG is omitted or nil, and
-toggle it if ARG is `toggle'; disable the mode otherwise.
+This is a minor mode.  If called interactively, toggle the
+`Highlight-Uses mode' mode.  If the prefix argument is positive,
+enable the mode, and if it is zero or negative, disable the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable
+the mode if ARG is nil, omitted, or is a positive number.
+Disable the mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `highlight-uses-mode'.
+
+The mode's hook is called both when the mode is enabled and when
+it is disabled.
 
 \(fn &optional ARG)" t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "highlight-uses-mode" '("highlight-uses-mode-")))
+(register-definition-prefixes "highlight-uses-mode" '("highlight-uses-mode-"))
 
 ;;;***
 
 ;;;### (autoloads nil "inf-haskell" "inf-haskell.el" (0 0 0 0))
 ;;; Generated autoloads from inf-haskell.el
 
-(let ((loads (get 'inferior-haskell 'custom-loads))) (if (member '"inf-haskell" loads) nil (put 'inferior-haskell 'custom-loads (cons '"inf-haskell" loads))))
-
 (autoload 'run-haskell "inf-haskell" "\
 Show the inferior-haskell buffer.  Start the process if needed." t nil)
 
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "inf-haskell" '("haskell-" "inf")))
+(register-definition-prefixes "inf-haskell" '("haskell-" "inf"))
 
 ;;;***
 
 ;;;### (autoloads nil "w3m-haddock" "w3m-haddock.el" (0 0 0 0))
 ;;; Generated autoloads from w3m-haddock.el
 
-(defface w3m-haddock-heading-face '((((class color)) :inherit highlight)) "\
-Face for quarantines." :group 'haskell)
-
-(if (fboundp 'register-definition-prefixes) (register-definition-prefixes "w3m-haddock" '("haskell-w3m-" "w3m-haddock-")))
+(register-definition-prefixes "w3m-haddock" '("haskell-w3m-" "w3m-haddock-"))
 
 ;;;***
 
